{
    * LoadData.java
    * Copyright (C) 1998 ymnk, JCraft Inc.
    *
    * This file is part of JSpringies, a mass and spring simulation system for Java
    *
    * JSpringies is free software; you can redistribute it and/or modify
    * it under the terms of the GNU General Public License as published by
    * the Free Software Foundation; either version 1, or (at your option)
    * any later version.
    *
    * JSpringies is distributed in the hope that it will be useful,
    * but WITHOUT ANY WARRANTY; without even the implied warranty of
    * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    * GNU General Public License for more details.
    *
    * You should have received a copy of the GNU General Public License
    * along with XSpringies; see the file COPYING.  If not, write to
    * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
    *

    JSpringies -- A mass and spring simulation system, written in pure Java
    http://www.jcraft.com/jspringies/

    Port description for games/xspringies
    http://www.freebsd.org/cgi/url.cgi?ports/games/xspringies/pkg-descr
    http://www.freebsd.org/cgi/pds.cgi?ports/games/xspringies
}

//   Delphi ɖ|

//  BlueLeaf1336    http://www.geocities.jp/fjtkt/

unit LoadData;

interface

uses
    SysUtils, Classes, dcraft;

type
//=============================================================================
//  TStringTokenizer

    TStringTokenizer = class(TObject)
    private
        FIndex: Integer;
        FToken: String;
        FText : String;
        function IsInLength(): Boolean;
        function IsBlank(): Boolean;
        procedure GetToken();
        procedure SkipBlank();
        procedure SetText(const AText: String);
    public
        constructor Create();
        destructor Destroy; override;
    	function NextToken(): String;
        function HasMoreTokens(): Boolean;
        function GetDouble(): Extended;
        function GetInt(): Integer;
        property Text: String read FText write SetText;
    end;

//=============================================================================
//  TLoadData in LoadData.java

    TLoadData = class(TObject)
    protected
        scan: TStringList;
        line, tokje: String;
        tok: TStringTokenizer;
        sp: TSpace;
        procedure masscommand();
        function mass_radius(const m: Extended): Integer;
        procedure spngcommand();
    public
        constructor Create();
        destructor Destroy(); override;
        function load(const _config_path: String; const _sp: TSpace): Boolean;
    end;

implementation

uses
    Math;

//=============================================================================
//  TStringTokenizer

//-----------------------------------------------------------------------------
//  RXgN^
constructor TStringTokenizer.Create();
begin
    SetText('');
end;

//-----------------------------------------------------------------------------
//  fXgN^
destructor TStringTokenizer.Destroy;
begin
    inherited;
end;

//-----------------------------------------------------------------------------
//  Zbg
procedure TStringTokenizer.SetText(const AText: String);
begin
    FText  := AText;	//앶Zbg
    FIndex := 0;    	//݈ʒu
    FToken := '';   	//Jgg[N
end;

//-----------------------------------------------------------------------------
//  ݈ʒuɎ܂Ă邩ǂ`FbN
function TStringTokenizer.IsInLength(): Boolean;
begin
    Result := (FIndex <= length(FText));
end;

//-----------------------------------------------------------------------------
//  ݈ʒu̕uNǂ`FbN
function TStringTokenizer.IsBlank(): Boolean;
const
    BLANK_CHAR = ' ';
begin
    Result := (Pos(FText[FIndex], BLANK_CHAR) > 0);
end;

//-----------------------------------------------------------------------------
//  uNǂݔ΂AP꒼ÖʒuɌ݈ʒui߂
procedure TStringTokenizer.SkipBlank();
begin
    while true do
    begin
		//݈ʒui߂
        Inc(FIndex);
        //񒷂𒴂Ăꍇ͏I
        if not IsInLength() then break;
		//uNȊOȂI
        if not IsBlank() then break;
    end;
		//݈ʒu߂
    Dec(FIndex);
end;

//-----------------------------------------------------------------------------
//  ݈ʒuȍ~̒P1oA݈ʒuPIʒu܂Ői߂
procedure TStringTokenizer.GetToken();
begin
	//uNǂݔ΂
    SkipBlank();
	//Jgg[N
    FToken := '';
    while true do
    begin
		//݈ʒui߂
        Inc(FIndex);
        //񒷂𒴂Ăꍇ͏I
        if not IsInLength() then break;
		//uNȂI
        if IsBlank() then break;
		//Jgg[NɌ݈ʒu̕A
        FToken := FToken + FText[FIndex];
    end;
	//݈ʒu߂
    Dec(FIndex);
end;

//-----------------------------------------------------------------------------
//  ̃g[N𓾂āA݈ʒui߂
function TStringTokenizer.NextToken(): String;
begin
    //g[Nǂ
	GetToken();
	//g[NԂ
    Result := FToken;
end;

//-----------------------------------------------------------------------------
//  ݈ʒuȍ~Ƀg[N݂邩ǂ𒲂ׂ
function TStringTokenizer.HasMoreTokens(): Boolean;
var
	CurrentIndex: Integer;
    CurrentToken: String;
begin
	//݈ʒuۑĂ
    CurrentIndex := FIndex;
	//Jgg[NۑĂ
    CurrentToken := FToken;
    //̃g[Nǂ
	GetToken();
	//g[N̗LԂ
    Result := (FToken <> '');
	//݈ʒu𕜌Ă
    FIndex := CurrentIndex;
	//Jgg[N𕜌Ă
    FToken := CurrentToken;
end;

//-----------------------------------------------------------------------------
//  ̃g[NduobleƂĎo
function TStringTokenizer.GetDouble(): Extended;
var
	token: String;
begin
	//̃g[N
	token := NextToken();
	//Extendedϊ
    Result := StrToFloat(token);
end;

//-----------------------------------------------------------------------------
//  ̃g[NIntegerƂĎo
function TStringTokenizer.GetInt(): Integer;
var
	token: String;
begin
	//̃g[N
	token := NextToken();
	//Integerϊ
    Result := StrToInt(token);
end;

//=============================================================================
//  TLoadData

//-----------------------------------------------------------------------------
constructor TLoadData.Create();
begin
    scan := TStringList.Create();
    tok := TStringTokenizer.Create();
end;
//-----------------------------------------------------------------------------
destructor TLoadData.Destroy();
begin
    scan.Free();
    tok.Free();
end;
//-----------------------------------------------------------------------------
function TLoadData.load(const _config_path: String; const _sp: TSpace): Boolean;
var
    i: Integer;
    frce: TForce;
    wall: TWall;
begin
    Result := False;
    if (not FileExists(_config_path)) then Exit;

    scan.LoadFromFile(_config_path);
    sp := _sp;

    for i := 0 to scan.Count - 1 do
    begin
        line := scan[i];

        //  ͊ɃZbg
        tok.Text := line;

        try
            //  g[NȂ(sȂ)̓XLbv
            if (not tok.HasMoreTokens()) then Continue;
            //  g[N̎o
            tokje := tok.NextToken();

            //  f[^ǂ݂ƑΉs
            if (tokje = 'mass') then
            begin
                masscommand();
            end
            else if (tokje = 'spng') then
            begin
                spngcommand();
            end
            else if (tokje = 'cmas') then
            begin
                sp.cur_mass := tok.GetDouble();
            end
            else if (tokje = 'elas') then
            begin
                sp.cur_rest := tok.GetDouble();
            end
            else if (tokje = 'kspr') then
            begin
                sp.cur_ks := tok.GetDouble();
            end
            else if (tokje = 'kdmp') then
            begin
                sp.cur_kd := tok.GetDouble();
            end
            else if (tokje = 'fixm') then
            begin
                sp.fix_mass := (tok.GetInt() <> 0);
            end
            else if (tokje = 'shws') then
            begin
                sp.show_spring := (tok.GetInt() <> 0);
            end
            else if (tokje = 'cent') then
            begin
                sp.center_id := tok.GetInt();
            end
            else if (tokje = 'frce') then
            begin
                case tok.GetInt() of
                  0: frce := sp.gravity;
                  1: frce := sp.cmass;
                  2: frce := sp.pattraction;
                  3: frce := sp.wall;
                else frce := nil;
                end;

                if (frce <> nil) then
                begin
                    frce.setEnable(tok.GetInt() > 0);
                    frce.setVal(tok.GetDouble());
                    frce.setMisc(tok.GetDouble());
                end
                else
                begin
                    tok.GetInt();
                    tok.GetDouble();
                    tok.GetDouble();
                end;
            end
            else if (tokje = 'visc') then
            begin
                sp.cur_visc := tok.GetDouble();
            end
            else if (tokje = 'stck') then
            begin
                sp.cur_stick := tok.GetDouble();
            end
            else if (tokje = 'step') then
            begin
                sp.cur_dt := tok.GetDouble();
            end
            else if (tokje = 'prec') then
            begin
                sp.cur_prec := tok.GetDouble();
            end
            else if (tokje = 'adpt') then
            begin
                sp.adaptive_step := (tok.GetInt() <> 0);
            end
            else if (tokje = 'gsnp') then
            begin
                sp.cur_gsnap := tok.GetDouble();
                sp.grid_snap := (tok.GetInt() <> 0);
            end
            else if (tokje = 'wall') then
            begin
                wall := sp.wall;
                wall.setTop(tok.GetInt() <> 0);
                wall.setLeft(tok.GetInt() <> 0);
                wall.setRight(tok.GetInt() <> 0);
                wall.setBottom(tok.GetInt() <> 0);
            end;
        except
		    raise Exception.CreateFmt('file: %s [line: %d] ɖ肪܂B', [_config_path, i]);
        end;
    end;

    Result := True;
end;
//-----------------------------------------------------------------------------
//  _ǉ鏈
procedure TLoadData.masscommand();
var
    id: Integer;
    foo: Boolean;
    x, y, vx, vy, mass, elastic: Extended;
    m: TMass;
begin
    id := tok.GetInt();
    x := tok.GetDouble();
    y := tok.GetDouble();
    vx := tok.GetDouble();
    vy := tok.GetDouble();
    mass := tok.GetDouble();
    elastic := tok.GetDouble();

    foo := (mass < 0);
    mass := Abs(mass);

    sp.addMass(id, x, y, vx, vy, mass, elastic);
    m := TMass(sp.masses.Last);

    if (foo) then m.Fixed();
    m.radius := mass_radius(m.mass);
end;
//-----------------------------------------------------------------------------
//  ʂɉē_̑傫߂
function TLoadData.mass_radius(const m: Extended): Integer;
var
    rad: Integer;
begin
    rad := Round(2 * Log10(4.0 * m + 1.0));
    //if rad < 1 then rad := 1;
    if rad < 4 then rad := 4;
    if rad > 64 then rad := 64;
    Result := Round(rad + sp.spthick / 2);
end;
//-----------------------------------------------------------------------------
//  ΂˂ǉ鏈
procedure TLoadData.spngcommand();
var
    i, id, m1, m2: Integer;
    kd, ks, restlen: Extended;
    mass1, mass2, mass: TMass;
begin
    id := tok.GetInt();
    m1 := tok.GetInt();
    m2 := tok.GetInt();
    ks := tok.GetDouble();
    kd := tok.GetDouble();
    restlen := tok.GetDouble();
    mass1 := nil;
    mass2 := nil;
    for i := 0 to sp.masses.Count - 1 do
    begin
        mass := TMass(sp.masses.Items[i]);
        if (m1 = mass.id) then mass1 := mass;
        if (m2 = mass.id) then mass2 := mass;
        if (mass1 <> nil) and (mass2 <> nil) then Break;
    end;
    if (mass1 <> nil) and (mass2 <> nil) then
    begin
        sp.addSpring(id, mass1, mass2, ks, kd, restlen);
    end;
end;

end.
