unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, dcraft, LoadData, Buttons;

type
    TTimerThread = class(TThread)
    protected
        procedure GetConfig();
        procedure UpdateForm();
        procedure Execute(); override;
    end;

type
  TMainForm = class(TForm)
    PboxSpace: TPaintBox;
    PnlControl: TPanel;
    cbgrv: TCheckBox;
    cbmass: TCheckBox;
    cbatt: TCheckBox;
    cbwall: TCheckBox;
    cbcsp: TCheckBox;
    LboxBrowse: TListBox;
    BtnStart: TBitBtn;
    BtnStop: TBitBtn;
    BtnVer: TBitBtn;
    cbas: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure PboxSpacePaint(Sender: TObject);
    procedure LboxBrowseClick(Sender: TObject);
    procedure BtnStartClick(Sender: TObject);
    procedure BtnStopClick(Sender: TObject);
    procedure LboxBrowseDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure BtnVerClick(Sender: TObject);
  private
    sp: TSpace;
    ld: TLoadData;
    FTimerThread: TTimerThread;
    FPrevPath: String;
  public

  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

uses
    Info;

//-----------------------------------------------------------------------------
procedure TTimerThread.GetConfig();
begin
    with MainForm do
    begin
        sp.gravity.setEnable(cbgrv.Checked);
        sp.cmass.setEnable(cbmass.Checked);
        sp.pattraction.setEnable(cbatt.Checked);
        sp.wall.setEnable(cbwall.Checked);
        sp.withcolor := cbcsp.Checked;
        sp.adaptive_step := cbas.Checked;
    end;
end;

//-----------------------------------------------------------------------------
procedure TTimerThread.UpdateForm();
begin
    with MainForm do
    begin
        sp.animate_obj();
        PboxSpace.Repaint();
    end;
end;

//-----------------------------------------------------------------------------
procedure TTimerThread.Execute();
begin
    while not Terminated do
    begin
        Synchronize(GetConfig);
        Synchronize(UpdateForm);
        Sleep(10);
    end;                                       
end;

//-----------------------------------------------------------------------------
//  t@CꗗԂ
procedure Dir(const ADir, AMask: String; AList: TStrings);
var
    LDir: String;
    LRec: TSearchRec;
begin
    LDir := IncludeTrailingPathDelimiter(ADir);
    if FindFirst(LDir + AMask, faAnyFile, LRec) = 0 then
    begin
        AList.Add(LDir + LRec.Name);
        while FindNext(LRec) = 0 do
        begin
	        AList.Add(LDir + LRec.Name);
        end;
        FindClose(LRec);
    end;
end;

//-----------------------------------------------------------------------------
procedure TMainForm.FormCreate(Sender: TObject);
begin
    Self.Caption := 
        ChangeFileExt(ExtractFileName(ParamStr(0)), '')
        + ' '
        + GetVersionInfo(FVERREQ_FileVersion, ParamStr(0))
        + ' is running.';

    PboxSpace.ControlStyle := PboxSpace.ControlStyle + [csOpaque];

    sp := TSpace.Create(PboxSpace.Width, PboxSpace.Height);
    ld := TLoadData.Create();

    LboxBrowse.Items.Clear();
    Dir(ExtractFilePath(ParamStr(0)) + 'lib\', '*.xsp', LboxBrowse.Items);
    PnlControl.Enabled := (LboxBrowse.Items.Count > 0);
    if (LboxBrowse.Items.Count > 0) then LboxBrowse.ItemIndex := 0;
    LboxBrowse.OnClick(LboxBrowse);

    FTimerThread := TTimerThread.Create(True);
end;

//-----------------------------------------------------------------------------
procedure TMainForm.FormDestroy(Sender: TObject);
begin
    FTimerThread.Terminate();
    FTimerThread.Free();

    sp.Free();
    ld.Free();
end;

//-----------------------------------------------------------------------------
procedure TMainForm.PboxSpacePaint(Sender: TObject);
var
    Lbox: TPaintBox;
    LBmp: TBitmap;
begin
    Lbox := TPaintBox(Sender);
    //sp.Draw(Lbox.Canvas, Lbox.Width, Lbox.Height);

    LBmp := TBitmap.Create();
    LBmp.Width := Lbox.Width;
    LBmp.Height := Lbox.Height;

    sp.Draw(LBmp.Canvas, LBmp.Width, LBmp.Height);

    BitBlt(Lbox.Canvas.Handle, 0, 0, LBmp.Width, LBmp.Height,
            LBmp.Canvas.Handle, 0, 0, SRCCOPY);

    LBmp.Free();
end;

//-----------------------------------------------------------------------------
procedure TMainForm.LboxBrowseClick(Sender: TObject);
var
    LPath: String;
    i: Integer;
    mass: TMass;
begin
    if (LboxBrowse.ItemIndex = -1) then Exit;

    LPath := LboxBrowse.Items.Strings[LboxBrowse.ItemIndex];

    if (FPrevPath = LPath) then Exit;

    FPrevPath := LPath;

    sp.clear();
    ld.load(LPath, sp);

    //  ݒeʂ
    cbgrv.Checked := sp.gravity.getEnable();
    cbmass.Checked := sp.cmass.getEnable();
    cbatt.Checked := sp.pattraction.getEnable();
    cbwall.Checked := sp.wall.getEnable();

    cbas.Checked := sp.adaptive_step;

    //  ^̎_ݒ
    mass := nil;
    if sp.center_id >= 0 then
    begin
	    for i := 0 to sp.masses.Count - 1 do
        begin
		    mass := TMass(sp.masses.Items[i]);
            if sp.center_id = mass.id then Break;
        end;
        sp.center_mass := mass;
    end;

    //  `
    PboxSpace.Repaint();
end;

//-----------------------------------------------------------------------------
procedure TMainForm.BtnStartClick(Sender: TObject);
begin
    if (LboxBrowse.ItemIndex = -1) then Exit;

    BtnStart.Enabled := False;
    BtnStop.Enabled := True;
    LboxBrowse.Enabled := False;

    FTimerThread.Resume();
end;

//-----------------------------------------------------------------------------
procedure TMainForm.BtnStopClick(Sender: TObject);
begin
    FTimerThread.Suspend();

    BtnStart.Enabled := True;
    BtnStop.Enabled := False;
    LboxBrowse.Enabled := True;
end;

//-----------------------------------------------------------------------------
procedure TMainForm.LboxBrowseDrawItem(Control: TWinControl;
  Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
    LListBox: TListBox;
    LCanvas: TCanvas;
    LRect: TRect;
    LCell: String;
begin
    LListBox := TListBox(Control);
    LCanvas := LListBox.Canvas;
    LRect := Rect;

    LCell := LListBox.Items.Strings[Index];
    LCell := ' ' + ExtractFileName(LCell);

    LCanvas.FillRect(LRect);

    DrawText(LCanvas.Handle, PChar(LCell), -1, LRect, DT_LEFT or DT_VCENTER);
end;

//-----------------------------------------------------------------------------
procedure TMainForm.BtnVerClick(Sender: TObject);
begin
    OpenInfoForm();
end;

end.
