{
    * *.java
    * Copyright (C) 1998 ymnk, JCraft Inc.
    *
    * This file is part of JSpringies, a mass and spring simulation system for Java
    *
    * JSpringies is free software; you can redistribute it and/or modify
    * it under the terms of the GNU General Public License as published by
    * the Free Software Foundation; either version 1, or (at your option)
    * any later version.
    *
    * JSpringies is distributed in the hope that it will be useful,
    * but WITHOUT ANY WARRANTY; without even the implied warranty of
    * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    * GNU General Public License for more details.
    *
    * You should have received a copy of the GNU General Public License
    * along with XSpringies; see the file COPYING.  If not, write to
    * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
    *

    JSpringies -- A mass and spring simulation system, written in pure Java
    http://www.jcraft.com/jspringies/

    Port description for games/xspringies
    http://www.freebsd.org/cgi/url.cgi?ports/games/xspringies/pkg-descr
    http://www.freebsd.org/cgi/pds.cgi?ports/games/xspringies
}

//   Delphi ɖ|

//  BlueLeaf1336    http://www.geocities.jp/fjtkt/
//  2006.09.16  * JSpringies ̃\[X擾
//  2006.09.16  |Jn
//  2006.09.17  JSpringies.java ȊOE|oOC
//              JSpringies.java ͉̏ʂ̖͕D悵ďu
//  2006.09.18  * xspringies ̃\[X擾
//              TSpace.accumulate_accel()  TSpring  TMass ւ̊^ TSpring.force() Ɉړ
//              Double => Extended ɕύX
//              TSpace.adaptive_runge_kutta() ǉ

unit dcraft;

interface

uses
    Classes, Graphics;

const
    radian: Extended = PI / 180.0;

    DT_MIN: Extended = 0.0001;
    DT_MAX: Extended = 0.5;
    CONSTR_KS: Extended = 1.0;
    CONSTR_KD: Extended = 1.0;
    DEF_TSTEP: Extended = 0.025;
    BOUNCE_FUDGE: Integer = 1;
    MAXCON: Integer = 1024;

    //  STICK_MAG = 1.0, means that a mass = 1.0 with gravity = 1.0 will
    //  remain stuck on a wall for all stickiness values > 1.0
    STICKMAG: Extended = 1.0;

    EPSILON: Extended = 0.00001;
type
//=============================================================================
//  TResStatus

    TResStatus = (rsALIVE, rsSELECTED, rsFIXED, rsTEMPFIXED);
    TResStatuses = set of TResStatus;

//=============================================================================
//  TResource in Resource.java

    TResource = class(TObject)
    protected
        status: TResStatuses;
    protected
        function Alivep(): Boolean;
        function Fixedp(): Boolean;
        function Selectedp(): Boolean;
        function Tempfixedp(): Boolean;
        function AliveNotFixedp(): Boolean;
    public
        id: Integer;
        constructor Create(const _id: Integer);
        procedure Alive();
        procedure Fixed();
        procedure Selected();
        procedure Tempfixed();
    end;

//=============================================================================
//  TMass in Mass.java

    TMass = class(TResource)
    public
        //  Current position, velocity, acceleration
        x, y: Extended;
        vx, vy: Extended;
        ax, ay: Extended;

        //  Mass and radius of mass
        mass, elastic: Extended;
        radius: Integer;

        //  Runge-Kutta temporary space
        cur_x, cur_y, cur_vx, cur_vy: Extended;
        old_x, old_y, old_vx, old_vy: Extended;
        test_x, test_y, test_vx, test_vy: Extended;
        k1x, k1y, k1vx, k1vy: Extended;
        k2x, k2y, k2vx, k2vy: Extended;
        k3x, k3y, k3vx, k3vy: Extended;
        k4x, k4y, k4vx, k4vy: Extended;
    public
        constructor Create(const _id: Integer; const _x, _y, _vx, _vy, _mass, _erastic: Extended);
        procedure save2Old();
        procedure save2Cur();
    end;

//=============================================================================
//  TSpring in Spring.java

    TSpring = class(TResource)
    public
        //  Ks, Kd and rest length of spring
        ks, kd: Extended;
        restlen: Extended;

        //  Connected to masses m1 and m2
        mass1, mass2: TMass;
    public
        constructor Create(const _id: Integer; const _mass1, _mass2: TMass;
                            const _ks, _kd, _restlen: Extended);
        procedure force();
    end;

//=============================================================================
//  TForce in Force.java

    TSpace = class;

    TForce = class(TObject)
    protected
        sp: TSpace;
        gval: Extended;
        gmisc: Extended;
        enable: Boolean;
    protected
        procedure force(); virtual; abstract;
    public
        constructor Create(const _s: TSpace; const _v, _m: Extended);
        function getVal(): Extended;
        function getMisc(): Extended;
        function getEnable(): Boolean;
        procedure setVal(const _v: Extended); virtual;
        procedure setMisc(const _m: Extended); virtual;  
        procedure setEnable(const _b: Boolean);
    end;

//=============================================================================
//  TGravity in Gravity.java

    TGravity = class(TForce)
    protected
        gx, gy: Extended;
    protected
        procedure update();
        procedure force(); override;
    public
        constructor Create(const _s: TSpace; const _v, _m: Extended);
        procedure setVal(const _v: Extended); override;
        procedure setMisc(const _m: Extended); override;
    end;

//=============================================================================
//  TCenterOfMass in CenterOfMass.java

    TCenterOfMass = class(TForce)
    protected
        ogx, ogy: Extended;
        procedure force(); override;
    public
        constructor Create(const _s: TSpace; const _v, _m: Extended);
    end;

//=============================================================================
//  TPointerAttraction in PointerAttraction.java

    TPointerAttraction = class(TForce)
    protected
        procedure force(); override;
    public
        constructor Create(const _s: TSpace; const _v, _m: Extended);
    end;

//=============================================================================
//  TWall in Wall.java

    TWall = class(TForce)
    protected
        width, height: Extended;
        top, left, right, bottom: Boolean;
        procedure force(); override;
    public
        constructor Create(const _s: TSpace; const _v, _m: Extended);
        procedure setTop(_b: Boolean);
        procedure setLeft(_b: Boolean);
        procedure setRight(_b: Boolean);
        procedure setBottom(_b: Boolean);
    end;

//=============================================================================
//  TSpace in Space.java

    TSpace = class
    protected
        num_since: Integer;
        time_elapsed: Extended;

        center_x: Extended;
        center_y: Extended;
        center_rad: Extended;
    public
        cur_mass, cur_rest: Extended;
        cur_ks, cur_kd: Extended;
        fix_mass, show_spring: Boolean;
        center_id: Integer;
        cur_gsnap: Extended;

        spthick: Integer;
        cur_visc, cur_stick: Extended;
        cur_dt, cur_prec: Extended;
        adaptive_step, grid_snap: Boolean;

        withcolor: Boolean;

        wall: TWall;
        gravity: TGravity;
        pattraction: TPointerAttraction;
        cmass: TCenterOfMass;

        springs: TList;
        masses: TList;

        width, height: Extended;
        center_mass: TMass;
    protected
        procedure accumulate_accel();
        procedure runge_kutta(const h: Extended; const testloc: Boolean);
        procedure adaptive_runge_kutta();
    public
        constructor Create(const _w, _h: Extended);
        destructor Destroy(); override;
        procedure clear();
        procedure addMass(const _id: Integer; const _x, _y, _vx, _vy, _mass, _elastic: Extended);
        procedure addSpring(const _id: Integer; const _mass1, _mass2: TMass;
                            const _ks, _kd, _restlen: Extended);
        function animate_obj(): Boolean;
        procedure Draw(Canvas: TCanvas; const W, H: Integer);
    end;

implementation

uses
    Math;

//=============================================================================
//  TResource

//-----------------------------------------------------------------------------
constructor TResource.Create(const _id: Integer);
begin
    id := _id;
    status := [rsALIVE];
end;
//-----------------------------------------------------------------------------
function TResource.Alivep(): Boolean;
begin
    Result := rsALIVE in status;
end;
//-----------------------------------------------------------------------------
function TResource.Fixedp(): Boolean;
begin
    Result := rsFIXED in status;
end;
//-----------------------------------------------------------------------------
function TResource.Selectedp(): Boolean;
begin
    Result := rsSELECTED in status;
end;
//-----------------------------------------------------------------------------
function TResource.Tempfixedp(): Boolean;
begin
    Result := rsTEMPFIXED in status;
end;
//-----------------------------------------------------------------------------
function TResource.AliveNotFixedp(): Boolean;
begin
    Result := Alivep() and (not Fixedp());
end;
//-----------------------------------------------------------------------------
procedure TResource.Alive();
begin
    status := status + [rsALIVE];
end;
//-----------------------------------------------------------------------------
procedure TResource.Fixed();
begin
    status := status + [rsFIXED];
end;
//-----------------------------------------------------------------------------
procedure TResource.Selected();
begin
    status := status + [rsSELECTED];
end;
//-----------------------------------------------------------------------------
procedure TResource.Tempfixed();
begin
    status := status + [rsTEMPFIXED];
end;

//=============================================================================
//  TMass

//-----------------------------------------------------------------------------
constructor TMass.Create(const _id: Integer; const _x, _y, _vx, _vy, _mass, _erastic: Extended);
begin
    inherited Create(_id);

    x  := _x;
    y  := _y;
    vx := _vx;
    vy := _vy;
    ax := 0.0;
    ay := 0.0;
    mass    := _mass;
    elastic := _erastic;
    radius := 0;
end;
//-----------------------------------------------------------------------------
procedure TMass.save2Old();
begin
    old_x  := x;
    old_y  := y;
    old_vx := vx;
    old_vy := vy;
end;
//-----------------------------------------------------------------------------
procedure TMass.save2Cur();
begin
    cur_x  := x;
    cur_y  := y;
    cur_vx := vx;
    cur_vy := vy;
end;

//=============================================================================
//  TSpring

//-----------------------------------------------------------------------------
constructor TSpring.Create(const _id: Integer; const _mass1, _mass2: TMass;
                    const _ks, _kd, _restlen: Extended);
begin
    inherited Create(_id);

    mass1 := _mass1;
    mass2 := _mass2;
    ks := _ks;
    kd := _kd;
    restlen := _restlen;
end;

//-----------------------------------------------------------------------------
procedure TSpring.force();
var
    dx, dy, force, forcex, forcey, mag, damp, m1, m2: Extended;
begin
    dx := mass1.x - mass2.x;
    dy := mass1.y - mass2.y;

    if ((dx <> 0) or (dy <> 0)) then
    begin
        mag := Sqrt(dx * dx + dy * dy);
        force := ks * (restlen - mag);

        if (kd <> 0.0) then
        begin
            damp := ((mass1.vx - mass2.vx) * dx + (mass1.vy - mass2.vy) * dy) / mag;
            force := force - (kd * damp);
        end;

        force  := force / mag;
        forcex := force * dx;
        forcey := force * dy;
        m1  := mass1.mass;
        m2  := mass2.mass;

        mass1.ax := mass1.ax + (forcex / m1);
        mass1.ay := mass1.ay + (forcey / m1);
        mass2.ax := mass2.ax - (forcex / m2);
        mass2.ay := mass2.ay - (forcey / m2);
    end;
end;

//=============================================================================
//  TForce

//-----------------------------------------------------------------------------
constructor TForce.Create(const _s: TSpace; const _v, _m: Extended);
begin
    sp := _s;
    gval  := _v;
    gmisc := _m;
    enable := False;
end;
//-----------------------------------------------------------------------------
procedure TForce.setVal(const _v: Extended);
begin
    gval := _v;
end;
//-----------------------------------------------------------------------------
procedure TForce.setMisc(const _m: Extended);
begin
    gmisc := _m;
end;
//-----------------------------------------------------------------------------
procedure TForce.setEnable(const _b: Boolean);
begin
    enable := _b;
end;
//-----------------------------------------------------------------------------
function TForce.getVal(): Extended;
begin
    Result := gval;
end;
//-----------------------------------------------------------------------------
function TForce.getMisc(): Extended;
begin
    Result := gmisc;
end;
//-----------------------------------------------------------------------------
function TForce.getEnable(): Boolean;
begin
    Result := enable;
end;

//=============================================================================
//  TGravity

//-----------------------------------------------------------------------------
constructor TGravity.Create(const _s: TSpace; const _v, _m: Extended);
begin
    inherited Create(_s, _v, _m);
    update();
end;
//-----------------------------------------------------------------------------
procedure TGravity.setVal(const _v: Extended);
begin
    inherited setVal(_v);
    update();
end;
//-----------------------------------------------------------------------------
procedure TGravity.setMisc(const _m: Extended);
begin
    inherited setMisc(_m);
    update();
end;
//-----------------------------------------------------------------------------
procedure TGravity.update();
var
    sin, cos: Extended;
begin
    SinCos(gmisc * radian, sin, cos);
    gx := gval * sin;
    gy := gval * cos;
end;
//-----------------------------------------------------------------------------
procedure TGravity.force();
var
    m: TMass;
    i: Integer;
begin
    for i := 0 to sp.masses.Count - 1 do
    begin
        m := TMass(sp.masses.Items[i]);
        if (m.AliveNotFixedp()) then
        begin
            m.ax := m.ax + gx;
            m.ay := m.ay + gy;
        end;
    end;
end;

//=============================================================================
//  TCenterOfMass

//-----------------------------------------------------------------------------
constructor TCenterOfMass.Create(const _s: TSpace; const _v, _m: Extended);
begin
    inherited Create(_s, _v, _m);
end;

//-----------------------------------------------------------------------------
procedure TCenterOfMass.force();
var
    mixix, mixiy, mivix, miviy, msum: Extended;
    i: Integer;
    m: TMass;
begin
    ogx := 0.0;
    ogy := 0.0;

    mixix := 0.0;
    mixiy := 0.0;
    mivix := 0.0;
    miviy := 0.0;
    msum  := 0.0;

    for i := 0 to sp.masses.Count - 1 do
    begin
        m := TMass(sp.masses.Items[i]);
        if ((m <> sp.center_mass) and (m.AliveNotFixedp())) then
        begin
            msum := msum + m.mass;
            mixix := mixix + (m.mass * m.x);
            mixiy := mixiy + (m.mass * m.y);
            mivix := mivix + (m.mass * m.vx);
            miviy := miviy + (m.mass * m.vy);
        end;
    end;

    if (msum > 0.0) then
    begin
        mixix := mixix / msum;
        mixiy := mixiy / msum;
        mivix := mivix / msum;
        miviy := miviy / msum;
        mixix := mixix - sp.center_x;
        mixiy := mixiy - sp.center_y;
        ogx := ogx - ((gval * mixix + gmisc * mivix) / msum);
        ogy := ogy - ((gval * mixiy + gmisc * miviy) / msum);
    end;
end;

//=============================================================================
//  TPointerAttraction

//-----------------------------------------------------------------------------
constructor TPointerAttraction.Create(const _s: TSpace; const _v, _m: Extended);
begin
    inherited Create(_s, _v, _m);
end;
//-----------------------------------------------------------------------------
procedure TPointerAttraction.force();
var
    m: TMass;
    center_x, center_y, center_rad: Extended;
    dx, dy, mag, fmag: Extended;
    foo, bar: Extended;
    i: Integer;
begin
    center_x := sp.center_x;
    center_y := sp.center_y;
    center_rad := sp.center_rad;

    for i := 0 to sp.masses.Count - 1 do
    begin
        m := TMass(sp.masses.Items[i]);
        if (m.AliveNotFixedp()) then
        begin
            dx := (center_x - m.x);
            dy := (center_y - m.y);
            mag := Sqrt(dx * dx + dy * dy);
            if (mag < m.radius + center_rad) then
            begin
                foo := m.radius + center_rad;
                bar := mag / foo;
                dx := dx * bar;
                dy := dy * bar;
                mag := foo;
            end;
            fmag := (gval / Power(mag, gmisc)) / mag;
            m.ax := m.ax + fmag * dx;
            m.ay := m.ay + fmag * dy;
        end;
    end;
end;

//=============================================================================
//  TWall in Wall.java

//-----------------------------------------------------------------------------
constructor TWall.Create(const _s: TSpace; const _v, _m: Extended);
begin
    inherited Create(_s, _v, _m);

    width  := _s.width;
    height := _s.height;
    top    := True;
    left   := True;
    right  := True;
    bottom := True;
end;
//-----------------------------------------------------------------------------
procedure TWall.setTop(_b: Boolean);
begin
    top := _b;
end;
//-----------------------------------------------------------------------------
procedure TWall.setLeft(_b: Boolean);
begin
    left := _b;
end;
//-----------------------------------------------------------------------------
procedure TWall.setRight(_b: Boolean);
begin
    right := _b;
end;
//-----------------------------------------------------------------------------
procedure TWall.setBottom(_b: Boolean);
begin
    bottom := _b;
end;
//-----------------------------------------------------------------------------
procedure TWall.force();
var
    dax, day, dist: Extended;
    m: TMass;
    i: Integer;
begin
    for i := 0 to sp.masses.Count - 1 do
    begin
        m := TMass(sp.masses.Items[i]);
        if (m.AliveNotFixedp()) then
        begin
            dax := 0;
            day := 0;    

            dist := m.x - m.radius;
            if (left and (dist >= 0)) then
            begin
                if (dist < 1) then dist := 1;
                dist := Power(dist, gmisc);
                dax := dax + (gval / dist);
            end;

            dist := width - m.radius - m.x;
            if (right and (dist >= 0)) then
            begin
                if (dist < 1) then dist := 1;
                dist := Power(dist, gmisc);
                dax := dax - (gval / dist);
            end;

            dist := height - m.radius - m.y;
            if (bottom and (dist >= 0)) then
            begin
                if (dist < 1) then dist := 1;
                dist := Power(dist, gmisc);
                day := day - (gval / dist);
            end;

            dist := m.y - m.radius;
            if (top and (dist >= 0)) then
            begin
                if (dist < 1) then dist := 1;
                dist := Power(dist, gmisc);
                day := day + (gval / dist);
            end;

            m.ax := m.ax + dax;
            m.ay := m.ay + day;
        end;
    end;
end;

//=============================================================================
//  TSpace

//-----------------------------------------------------------------------------
constructor TSpace.Create(const _w, _h: Extended);
begin
    cur_mass := 1.0;
    cur_rest := 1.0;
    cur_ks   := 1.0;
    cur_kd   := 1.0;
    fix_mass    := False;
    show_spring := True;
    center_id := -1;
    spthick   := 0;

    cur_visc  := 0.0;
    cur_stick := 0.0;
    cur_dt    := DEF_TSTEP;
    cur_prec  := 1.0;
    adaptive_step := False;
    grid_snap := False;
    cur_gsnap := 20.0;

    center_mass := nil;

    num_since    := 0;
    time_elapsed := 0.0;

    center_x   := 0.0;
    center_y   := 0.0;
    center_rad := 0.0;

    width  := _w;
    height := _h;
    
    gravity     := TGravity.Create(Self, 10.0, 0.0);
    cmass       := TCenterOfMass.Create(Self, 5.0, 2.0);
    pattraction := TPointerAttraction.Create(Self, 10.0, 0.0);
    wall        := TWall.Create(Self, 10000.0, 1.0);

    springs := TList.Create();
    masses  := TList.Create();
end;
//-----------------------------------------------------------------------------
destructor TSpace.Destroy();
begin
    gravity.Free();
    cmass.Free();
    pattraction.Free();
    wall.Free();

    Clear();
    springs.Free();
    masses.Free();

    inherited Destroy();
end;
//-----------------------------------------------------------------------------
procedure TSpace.clear();
begin
    while (springs.Count > 0) do
    begin
        TSpring(springs.Items[0]).Free();
        springs.Delete(0);
    end;

    while (masses.Count > 0) do
    begin
        TMass(masses.Items[0]).Free();
        masses.Delete(0);
    end;
end;
//-----------------------------------------------------------------------------
procedure TSpace.addMass(const _id: Integer; const _x, _y, _vx, _vy, _mass, _elastic: Extended);
var
    m: TMass;
begin
    m := TMass.Create(_id, _x, _y, _vx, _vy, _mass, _elastic);
    masses.Add(m);
end;
//-----------------------------------------------------------------------------
procedure TSpace.addSpring(const _id: Integer; const _mass1, _mass2: TMass;
                            const _ks, _kd, _restlen: Extended);
var
    spring: TSpring;
begin
    spring := TSpring.Create(_id, _mass1, _mass2, _ks, _kd, _restlen);
    springs.Add(spring);
end;
//-----------------------------------------------------------------------------
procedure TSpace.accumulate_accel();
var
    gx, gy, ogx, ogy: Extended;
    i: Integer;
    m: TMass;
    s: TSpring;
//    m1, m2: TMass;
//    dx, dy, force, forcex, forcey, mag, damp, mass1, mass2: Extended;
begin
    gx := 0.0;
    gy := 0.0;
    ogx := 0.0;
    ogy := 0.0;

    center_x := width  / 2.0;
    center_y := height / 2.0;
    center_rad := 1.0;

    if (masses.Count = 0) then Exit;

    if (center_mass <> nil) then
    begin
        if (center_mass.Alivep()) then
        begin
            center_x := center_mass.x;
            center_y := center_mass.x;
        end;
    end;

    //  Do gravity
    if (gravity.getEnable()) then
    begin
        gx := gravity.gx;
        gy := gravity.gy;
    end;

    //  Keep center of mass in the middle force
    if (cmass.getEnable()) then
    begin
        cmass.force();
        ogx := cmass.ogx;
        ogy := cmass.ogy;
    end;

    //  Apply Gravity, CM and air drag to all masses
    for i := 0 to masses.Count - 1 do
    begin
        m := TMass(masses.Items[i]);
        if (m.AliveNotFixedp()) then
        begin
            //  Do viscous drag
            if (m.id <> center_id) then
            begin
                m.ax := gx + ogx - (cur_visc * m.vx);
                m.ay := gy + ogy - (cur_visc * m.vy);
            end
            else
            begin
                m.ax := gx - (cur_visc * m.vx);
                m.ay := gy - (cur_visc * m.vy);
            end;
        end;
    end;

    //  Do point attraction force
    if (pattraction.getEnable()) then
    begin
        pattraction.force();
    end;

    //  Wall attract/repel force
    if (wall.getEnable()) then
    begin
        wall.force();
    end;

    //  Spring compression/damping effects on masses
    if (springs.Count > 0) then
    begin
        for i := 0 to springs.Count - 1 do
        begin
            s := TSpring(springs.Items[i]);
            if (s.Alivep()) then
            begin
                s.force();
            end;
        end;
    end;
end;
//-----------------------------------------------------------------------------
procedure TSpace.runge_kutta(const h: Extended; const testloc: Boolean);
var
    i: Integer;
    m: TMass;
begin
    if (masses.Count = 0) then Exit;

    accumulate_accel();

    //  k1 step
    for i := 0 to masses.Count - 1 do
    begin
        m := TMass(masses.Items[i]);

        if (m.AliveNotFixedp()) then
        begin
            m.save2Cur();

            m.k1x  := m.vx * h;
            m.k1y  := m.vy * h;
            m.k1vx := m.ax * h;
            m.k1vy := m.ay * h;

            m.x  := m.cur_x  + (m.k1x  / 2.0);
            m.y  := m.cur_y  + (m.k1y  / 2.0);
            m.vx := m.cur_vx + (m.k1vx / 2.0);
            m.vy := m.cur_vy + (m.k1vy / 2.0);
        end;
    end;
    accumulate_accel();

    //  k2 step
    for i := 0 to masses.Count - 1 do
    begin
        m := TMass(masses.Items[i]);

        if (m.AliveNotFixedp()) then
        begin
            m.k2x  := m.vx * (h / 2.0);
            m.k2y  := m.vy * (h / 2.0);
            m.k2vx := m.ax * (h / 2.0);
            m.k2vy := m.ay * (h / 2.0);

            m.x  := m.cur_x  + (m.k2x  / 2.0);
            m.y  := m.cur_y  + (m.k2y  / 2.0);
            m.vx := m.cur_vx + (m.k2vx / 2.0);
            m.vy := m.cur_vy + (m.k2vy / 2.0);
        end;
    end;
    accumulate_accel();

    //  k3 step
    for i := 0 to masses.Count - 1 do
    begin
        m := TMass(masses.Items[i]);

        if (m.AliveNotFixedp()) then
        begin
            m.k3x  := m.vx * (h / 2.0);
            m.k3y  := m.vy * (h / 2.0);
            m.k3vx := m.ax * (h / 2.0);
            m.k3vy := m.ay * (h / 2.0);

            m.x  := m.cur_x  + m.k3x;
            m.y  := m.cur_y  + m.k3y;
            m.vx := m.cur_vx + m.k3vx;
            m.vy := m.cur_vy + m.k3vy;
        end;
    end;
    accumulate_accel();

    //  k4 step
    for i := 0 to masses.Count - 1 do
    begin
        m := TMass(masses.Items[i]);

        if (m.AliveNotFixedp()) then
        begin
            m.k4x  := m.vx * h;
            m.k4y  := m.vy * h;
            m.k4vx := m.ax * h;
            m.k4vy := m.ay * h;
        end;
    end;

    //  Find next position
    for i := 0 to masses.Count - 1 do
    begin
        m := TMass(masses.Items[i]);

        if (m.AliveNotFixedp()) then
        begin
            if (testloc) then
            begin
                m.test_x  := m.cur_x  + (m.k1x  / 2.0 + m.k2x  + m.k3x  + m.k4x  / 2.0) / 3.0;
                m.test_y  := m.cur_y  + (m.k1y  / 2.0 + m.k2y  + m.k3y  + m.k4y  / 2.0) / 3.0;
                m.test_vx := m.cur_vx + (m.k1vx / 2.0 + m.k2vx + m.k3vx + m.k4vx / 2.0) / 3.0;
                m.test_vy := m.cur_vy + (m.k1vy / 2.0 + m.k2vy + m.k3vy + m.k4vy / 2.0) / 3.0;
            end
            else
            begin
                m.x  := m.cur_x  + (m.k1x  / 2.0 + m.k2x  + m.k3x  + m.k4x  / 2.0) / 3.0;
                m.y  := m.cur_y  + (m.k1y  / 2.0 + m.k2y  + m.k3y  + m.k4y  / 2.0) / 3.0;
                m.vx := m.cur_vx + (m.k1vx / 2.0 + m.k2vx + m.k3vx + m.k4vx / 2.0) / 3.0;
                m.vy := m.cur_vy + (m.k1vy / 2.0 + m.k2vy + m.k3vy + m.k4vy / 2.0) / 3.0;
            end;
        end;
    end;
end;
//-----------------------------------------------------------------------------
procedure TSpace.adaptive_runge_kutta();
var
    i: Integer;
    err, maxerr: Extended;
    m: TMass;
begin
    while (True) do
    begin
        if (cur_dt > DT_MAX) then cur_dt := DT_MAX;
        if (cur_dt < DT_MIN) then cur_dt := DT_MIN;

        runge_kutta(cur_dt / 2.0, False);
        runge_kutta(cur_dt / 2.0, True);

        for i := 0 to masses.Count - 1 do
        begin
            m := TMass(masses.Items[i]);
            if (m.AliveNotFixedp()) then
            begin
                m.x  := m.old_x;
                m.y  := m.old_y;
                m.vx := m.old_vx;
                m.vy := m.old_vy;
            end;
        end;
        runge_kutta(cur_dt, False);

        //  Find error
        maxerr := 0.00001;
        for i := 0 to masses.Count - 1 do
        begin
            m := TMass(masses.Items[i]);
            if (m.AliveNotFixedp()) then
            begin
                err := Abs(m.x  - m.test_x)  + Abs(m.y  - m.test_y)
                     + Abs(m.vx - m.test_vx) + Abs(m.vy - m.test_vy);
                if (err > maxerr) then maxerr := err;
            end;
        end;

        //  Fudgy scale factor -- user controlled
        maxerr := maxerr / cur_prec;

        if (maxerr < 1.0) then
        begin                              
            cur_dt := cur_dt * 0.9 * Exp(- Ln(maxerr) / 8.0);
        end
        else
        begin
            if (cur_dt > DT_MIN) then
            begin
                for i := 0 to masses.Count - 1 do
                begin
                    m := TMass(masses.Items[i]);
                    if (m.AliveNotFixedp()) then
                    begin
                        m.x  := m.old_x;
                        m.y  := m.old_y;
                        m.vx := m.old_vx;
                        m.vy := m.old_vy;
                    end;
                end;
                cur_dt := cur_dt * 0.9 * Exp(- Ln(maxerr) / 4.0);
                //adaptive_runge_kutta();
                Continue;
            end;
        end;

        Break;
    end;
end;
//-----------------------------------------------------------------------------
var
    num_since: Integer = 0;
    time_elapsed: Extended = 0.0;
//-----------------------------------------------------------------------------
function TSpace.animate_obj(): Boolean;
var
    i: Integer;
    stick_mag: Extended;
    m: TMass;
    s: TSpring;
    stick: Extended;
    any_spring: Boolean;
begin
    Result := False;

    if (masses.Count = 0) then Exit;

    //  Save initial values
    for i := 0 to masses.Count - 1 do
    begin
        m := TMass(masses.Items[i]);
        if (m.AliveNotFixedp()) then
        begin
            m.save2Old();
        end;
    end;

    if (adaptive_step) then
    begin
        any_spring := False;
        for i := 0 to springs.Count - 1 do
        begin
            s := TSpring(springs.Items[i]);
            if (s.Alivep()) then
            begin
                any_spring := True;
                Break;
            end;
        end;

        if (any_spring) then
        begin
            adaptive_runge_kutta();
        end
        else
        begin
            cur_dt := DEF_TSTEP;
            runge_kutta(cur_dt, false);
        end;
    end
    else
    begin
        runge_kutta(cur_dt, false);
    end;

    //runge_kutta(cur_dt, False);

    stick_mag := STICKMAG * cur_dt * cur_stick;
    stick     := STICKMAG * cur_stick;

    for i := 0 to masses.Count - 1 do
    begin
        m := TMass(masses.Items[i]);
        if (m.AliveNotFixedp()) then
        begin
            //  Check if stuck to a wall
            //if ((m.old_vx = 0.0) and (m.old_vy = 0.0)) then
            if ((Abs(m.old_vx) < EPSILON) and (Abs(m.old_vy) < EPSILON)) then
            begin
                //  Left or right wall
                if ((wall.left and (Abs(m.old_x - m.radius) < 0.5)) or
                  (wall.right and (Abs(m.old_x - width + m.radius) < 0.5))) then
                begin
                    if (Abs(m.vx) < (stick_mag / m.mass)) then
                    begin
                        m.vx := 0.0;
                        m.vy := 0.0;
                        m.x  := m.old_x;
                        m.y  := m.old_y;
                        Continue;
                    end;
                end
                //  Top or bottom wall
                else if ((wall.bottom and (Abs(m.old_y - m.radius) < 0.5)) or
                         (wall.top and (Abs(m.old_y - height + m.radius) < 0.5))) then
                begin
                    if (Abs(m.vy) < (stick_mag / m.mass)) then
                    begin
                        m.vx := 0.0;
                        m.vy := 0.0;
                        m.x  := m.old_x;
                        m.y  := m.old_y;
                        Continue;
                    end;
                end;
            end;

            //  Bounce off left or right wall
            if (wall.left and (m.x < m.radius) and (m.old_x >= m.radius)) then
            begin
                m.x := m.radius;
                if (m.vx < 0) then
                begin
                    m.vx := - m.vx * m.elastic;
                    m.vy :=   m.vy * m.elastic;

                    //  Get stuck if not going fast enough
                    if (m.vx > 0) then
                    begin
                        m.vx := m.vx - (stick / m.mass);
                        if (m.vx < 0) then
                        begin
                            m.vx := 0;
                            m.vy := 0;
                        end;
                    end;
                end;
            end
            else if (wall.right and (m.x > width - m.radius) and (m.old_x <= width - m.radius)) then
            begin
                m.x := width - m.radius;

                if (m.vx > 0) then
                begin
                    m.vx := - m.vx * m.elastic;
                    m.vy :=   m.vy * m.elastic;

                    //  Get stuck if not going fast enough
                    if (m.vx < 0) then
                    begin
                        m.vx := m.vx + (stick / m.mass);
                        if (m.vx > 0) then
                        begin
                            m.vx := 0;
                            m.vy := 0;
                        end;
                    end;
                end;
            end;


            //  Stick to top or bottom wall
            if (wall.top and (m.y < m.radius) and (m.old_y >= m.radius)) then
            begin
                m.y := m.radius;

                if (m.vy < 0) then
                begin
                    m.vy := - m.vy * m.elastic;
                    m.vx :=   m.vx * m.elastic;

                    //  Get stuck if not going fast enough
                    if (m.vy > 0) then
                    begin
                        m.vy := m.vy - (stick / m.mass);
                        if (m.vy < 0) then
                        begin
                            m.vx := 0;
                            m.vy := 0;
                        end;
                    end;
                end;
            end
            else if (wall.bottom and (m.y > (height - m.radius)) and
                     (m.old_y <= (height - m.radius))) then
            begin
                m.y := height - m.radius;
                if (m.vy > 0) then
                begin
                    m.vy := - m.vy * m.elastic;
                    m.vx :=   m.vx * m.elastic;

                    //  Get stuck if not going fast enough
                    if (m.vy < 0) then
                    begin
                        m.vy := m.vy + (stick / m.mass);
                        if (m.vy > 0) then
                        begin
                            m.vx := 0;
                            m.vy := 0;
                        end;
                    end;
                end;
            end;
        end;
    end;

    //  ȉAxspringies 
    time_elapsed := time_elapsed + cur_dt;

    if (time_elapsed > 0.05) then
    begin
        time_elapsed := time_elapsed - 0.05;
        num_since := 0;
        Result := True;
        Exit;
    end;

    Inc(num_since);

    if (num_since > 8) then
    begin
        num_since := 0;
        Result := True;
        Exit;
    end;

    Result := True;
end;

//-----------------------------------------------------------------------------
//  IuWFNg̕`
procedure TSpace.Draw(Canvas: TCanvas; const W, H: Integer);
var
    mass: TMass;
    m1, m2: TMass;
    spring: TSpring;
    foo, dx, dy: Extended;
    i: integer;
    tx, ty: Integer;
begin
    //  wihԂ
    Canvas.Brush.Color := clBlack;
    Canvas.FillRect(Rect(0, 0, W, H));

    //  _`
    Canvas.Brush.Color := clWhite;
    Canvas.Pen.Color := clWhite;
    
    for i := 0 to masses.Count - 1 do
    begin
        mass := masses[i];
        //Canvas.Ellipse(
        //        Round(mass.x - mass.radius / 2),
        //        Round(mass.y - mass.radius / 2),
        //        Round(mass.x + mass.radius / 2),
        //        Round(mass.y + mass.radius / 2)
        //);

        tx := Round(mass.x - mass.radius / 2);
        ty := Round(mass.y - mass.radius / 2);
        Canvas.Ellipse(tx, ty, tx + mass.radius, ty + mass.radius);
    end;

    //  ΂˕`
    for i := 0 to springs.Count - 1 do
    begin
        spring := springs[i];
        m1 := spring.mass1;
        m2 := spring.mass2;
        if withcolor then
        begin
            dx := m1.x - m2.x;
            dy := m1.y - m2.y;
            foo := Sqrt(dx * dx + dy * dy);
            foo := foo - spring.restlen;
            if Abs(foo) < 0.001 then
            begin
                Canvas.Pen.Color := clWhite;
            end
            else if (foo < 0) then
            begin
                Canvas.Pen.Color := clBlue;
            end
            else
            begin
                Canvas.Pen.Color := clRed;
            end;
        end
        else
        begin
            Canvas.Pen.Color := clWhite;
        end;
        Canvas.MoveTo(Round(m1.x), Round(m1.y));
        Canvas.LineTo(Round(m2.x), Round(m2.y));
        Canvas.Pen.Color := clWhite;
    end;
end;

end.
